package sun.management.snmp.jvminstr;

import java.lang.management.MemoryManagerMXBean;
import java.util.TreeMap;
import java.util.Collections;
import java.lang.management.MemoryPoolMXBean;
import java.util.HashMap;
import sun.management.snmp.util.SnmpCachedData;
import com.sun.jmx.mbeanserver.Util;
import java.util.Map;
import sun.management.snmp.util.JvmContextFactory;
import com.sun.jmx.snmp.SnmpStatusException;
import com.sun.jmx.snmp.SnmpOid;
import sun.management.snmp.util.SnmpTableHandler;
import com.sun.jmx.snmp.agent.SnmpStandardObjectServer;
import com.sun.jmx.snmp.agent.SnmpMib;
import sun.management.snmp.util.MibLogger;
import sun.management.snmp.util.SnmpTableCache;
import java.io.Serializable;
import sun.management.snmp.jvmmib.JvmMemMgrPoolRelTableMeta;

public class JvmMemMgrPoolRelTableMetaImpl extends JvmMemMgrPoolRelTableMeta implements Serializable
{
    static final long serialVersionUID = 1896509775012355443L;
    protected SnmpTableCache cache;
    private transient JvmMemManagerTableMetaImpl managers;
    private transient JvmMemPoolTableMetaImpl pools;
    static final MibLogger log;
    
    public JvmMemMgrPoolRelTableMetaImpl(SnmpMib snmpMib, SnmpStandardObjectServer snmpStandardObjectServer) {
        super(snmpMib, snmpStandardObjectServer);
        this.managers = null;
        this.pools = null;
        this.cache = new JvmMemMgrPoolRelTableMetaImpl.JvmMemMgrPoolRelTableCache(this, ((JVM_MANAGEMENT_MIB_IMPL)snmpMib).validity());
    }
    
    private final JvmMemManagerTableMetaImpl getManagers(SnmpMib snmpMib) {
        if (this.managers == null) {
            this.managers = (JvmMemManagerTableMetaImpl)snmpMib.getRegisteredTableMeta("JvmMemManagerTable");
        }
        return this.managers;
    }
    
    private final JvmMemPoolTableMetaImpl getPools(SnmpMib snmpMib) {
        if (this.pools == null) {
            this.pools = (JvmMemPoolTableMetaImpl)snmpMib.getRegisteredTableMeta("JvmMemPoolTable");
        }
        return this.pools;
    }
    
    protected SnmpTableHandler getManagerHandler(Object o) {
        return this.getManagers(this.theMib).getHandler(o);
    }
    
    protected SnmpTableHandler getPoolHandler(Object o) {
        return this.getPools(this.theMib).getHandler(o);
    }
    
    protected SnmpOid getNextOid(Object o) throws SnmpStatusException {
        return this.getNextOid(null, o);
    }
    
    protected SnmpOid getNextOid(SnmpOid obj, Object o) throws SnmpStatusException {
        int debugOn = JvmMemMgrPoolRelTableMetaImpl.log.isDebugOn();
        if (!(debugOn == 0)) {
            JvmMemMgrPoolRelTableMetaImpl.log.debug("getNextOid", "previous=" + obj);
        }
        SnmpTableHandler handler = this.getHandler(o);
        if (handler == null) {
            if (!(debugOn == 0)) {
                JvmMemMgrPoolRelTableMetaImpl.log.debug("getNextOid", "handler is null!");
            }
            throw new SnmpStatusException(224);
        }
        else {
            SnmpOid next = handler.getNext(obj);
            if (!(debugOn == 0)) {
                JvmMemMgrPoolRelTableMetaImpl.log.debug("getNextOid", "next=" + next);
            }
            if (next == null) {
                throw new SnmpStatusException(224);
            }
            return next;
        }
    }
    
    protected boolean contains(SnmpOid snmpOid, Object o) {
        SnmpTableHandler handler = this.getHandler(o);
        return handler != null && handler.contains(snmpOid);
    }
    
    public Object getEntry(SnmpOid snmpOid) throws SnmpStatusException {
        if (snmpOid == null || snmpOid.getLength() < 2) {
            throw new SnmpStatusException(224);
        }
        Map<String, JvmMemMgrPoolRelEntryImpl> userData = JvmContextFactory.getUserData();
        long oidArc = snmpOid.getOidArc(0);
        long oidArc2 = snmpOid.getOidArc(1);
        String s = (userData == null) ? null : ("JvmMemMgrPoolRelTable.entry." + oidArc + "." + oidArc2);
        if (userData != null) {
            Object value = userData.get(s);
            if (value != null) {
                return value;
            }
        }
        SnmpTableHandler handler = this.getHandler(userData);
        if (handler == null) {
            throw new SnmpStatusException(224);
        }
        Object data = handler.getData(snmpOid);
        if (!(data instanceof JvmMemMgrPoolRelEntryImpl)) {
            throw new SnmpStatusException(224);
        }
        JvmMemMgrPoolRelEntryImpl jvmMemMgrPoolRelEntryImpl = (JvmMemMgrPoolRelEntryImpl)data;
        if (userData != null && jvmMemMgrPoolRelEntryImpl != null) {
            userData.put(s, jvmMemMgrPoolRelEntryImpl);
            return jvmMemMgrPoolRelEntryImpl;
        }
        return jvmMemMgrPoolRelEntryImpl;
    }
    
    protected SnmpTableHandler getHandler(Object x) {
        Map<String, SnmpTableHandler> map;
        if ((x instanceof Map)) {
            map = (Map)Util.cast(x);
        }
        else {
            map = null;
        }
        if (map != null) {
            SnmpTableHandler snmpTableHandler = (SnmpTableHandler)map.get("JvmMemMgrPoolRelTable.handler");
            if (snmpTableHandler != null) {
                return snmpTableHandler;
            }
        }
        SnmpTableHandler tableHandler = this.cache.getTableHandler();
        if (map != null && tableHandler != null) {
            map.put("JvmMemMgrPoolRelTable.handler", tableHandler);
            return tableHandler;
        }
        return tableHandler;
    }
    
    static {
        log = new MibLogger(JvmMemMgrPoolRelTableMetaImpl.class);
    }
    
    private static class JvmMemMgrPoolRelTableCache extends SnmpTableCache
    {
        static final long serialVersionUID = 6059937161990659184L;
        private final JvmMemMgrPoolRelTableMetaImpl meta;
        
        JvmMemMgrPoolRelTableCache(JvmMemMgrPoolRelTableMetaImpl meta, long validity) {
            this.validity = validity;
            this.meta = meta;
        }
        
        public SnmpTableHandler getTableHandler() {
            return this.getTableDatas(JvmContextFactory.getUserData());
        }
        
        private static Map<String, SnmpOid> buildPoolIndexMap(SnmpTableHandler snmpTableHandler) {
            if ((snmpTableHandler instanceof SnmpCachedData)) {
                return buildPoolIndexMap((SnmpCachedData)snmpTableHandler);
            }
            HashMap<String, SnmpOid> hashMap = new HashMap();
            SnmpOid next = null;
            while ((next = snmpTableHandler.getNext(next)) != null) {
                MemoryPoolMXBean memoryPoolMXBean = (MemoryPoolMXBean)snmpTableHandler.getData(next);
                if (memoryPoolMXBean == null) {
                    continue;
                }
                String name = memoryPoolMXBean.getName();
                if (name == null) {
                    continue;
                }
                hashMap.put(name, next);
            }
            return hashMap;
        }
        
        private static Map<String, SnmpOid> buildPoolIndexMap(SnmpCachedData snmpCachedData) {
            if (snmpCachedData == null) {
                return Collections.emptyMap();
            }
            SnmpOid[] indexes = snmpCachedData.indexes;
            Object[] datas = snmpCachedData.datas;
            int length = indexes.length;
            HashMap<String, SnmpOid> hashMap = new HashMap(length);
            for (int i = 0; i < length; ++i) {
                SnmpOid snmpOid = indexes[i];
                if (snmpOid != null) {
                    MemoryPoolMXBean memoryPoolMXBean = (MemoryPoolMXBean)datas[i];
                    if (memoryPoolMXBean != null) {
                        String name = memoryPoolMXBean.getName();
                        if (name != null) {
                            hashMap.put(name, snmpOid);
                        }
                    }
                }
            }
            return hashMap;
        }
        
        protected SnmpCachedData updateCachedDatas(Object o) {
            SnmpTableHandler managerHandler = this.meta.getManagerHandler(o);
            SnmpTableHandler poolHandler = this.meta.getPoolHandler(o);
            long currentTimeMillis = System.currentTimeMillis();
            Map<String, SnmpOid> buildPoolIndexMap = buildPoolIndexMap(poolHandler);
            TreeMap treeMap = new TreeMap(SnmpCachedData.oidComparator);
            this.updateTreeMap(treeMap, o, managerHandler, poolHandler, buildPoolIndexMap);
            return new SnmpCachedData(currentTimeMillis, treeMap);
        }
        
        protected String[] getMemoryPools(Object x, MemoryManagerMXBean memoryManagerMXBean, long lng) {
            String string = "JvmMemManager." + lng + ".getMemoryPools";
            String[] memoryPoolNames = null;
            if ((x instanceof Map)) {
                memoryPoolNames = (String[])(String[])((Map)x).get(string);
                if (memoryPoolNames != null) {
                    return memoryPoolNames;
                }
            }
            if (memoryManagerMXBean != null) {
                memoryPoolNames = memoryManagerMXBean.getMemoryPoolNames();
            }
            if (memoryPoolNames != null && x instanceof Map) {
                ((Map)Util.cast(x)).put(string, memoryPoolNames);
                return memoryPoolNames;
            }
            return memoryPoolNames;
        }
        
        protected void updateTreeMap(TreeMap<SnmpOid, Object> treeMap, Object o, MemoryManagerMXBean memoryManagerMXBean, SnmpOid obj, Map<String, SnmpOid> map) {
            long oidArc;
            try {
                oidArc = obj.getOidArc(0);
            }
            catch (SnmpStatusException ex) {
                JvmMemMgrPoolRelTableMetaImpl.log.debug("updateTreeMap", "Bad MemoryManager OID index: " + obj);
                JvmMemMgrPoolRelTableMetaImpl.log.debug("updateTreeMap", ex);
                return;
            }
            String[] memoryPools = this.getMemoryPools(o, memoryManagerMXBean, oidArc);
            if (memoryPools == null || memoryPools.length < 1) {
                return;
            }
            String name = memoryManagerMXBean.getName();
            for (int i = 0; i < memoryPools.length; ++i) {
                String s = memoryPools[i];
                if (s != null) {
                    SnmpOid obj2 = (SnmpOid)map.get(s);
                    if (obj2 != null) {
                        long oidArc2;
                        try {
                            oidArc2 = obj2.getOidArc(0);
                        }
                        catch (SnmpStatusException ex2) {
                            JvmMemMgrPoolRelTableMetaImpl.log.debug("updateTreeMap", "Bad MemoryPool OID index: " + obj2);
                            JvmMemMgrPoolRelTableMetaImpl.log.debug("updateTreeMap", ex2);
                            continue;
                        }
                        treeMap.put(new SnmpOid(new long[] { oidArc, oidArc2 }), new JvmMemMgrPoolRelEntryImpl(name, s, (int)oidArc, (int)oidArc2));
                    }
                }
            }
            return;
        }
        
        protected void updateTreeMap(TreeMap<SnmpOid, Object> treeMap, Object o, SnmpTableHandler snmpTableHandler, SnmpTableHandler snmpTableHandler2, Map<String, SnmpOid> map) {
            if ((snmpTableHandler instanceof SnmpCachedData)) {
                this.updateTreeMap(treeMap, o, (SnmpCachedData)snmpTableHandler, snmpTableHandler2, map);
                return;
            }
            SnmpOid next = null;
            while ((next = snmpTableHandler.getNext(next)) != null) {
                MemoryManagerMXBean memoryManagerMXBean = (MemoryManagerMXBean)snmpTableHandler.getData(next);
                if (memoryManagerMXBean == null) {
                    continue;
                }
                this.updateTreeMap(treeMap, o, memoryManagerMXBean, next, map);
            }
            return;
        }
        
        protected void updateTreeMap(TreeMap<SnmpOid, Object> treeMap, Object o, SnmpCachedData snmpCachedData, SnmpTableHandler snmpTableHandler, Map<String, SnmpOid> map) {
            SnmpOid[] indexes = snmpCachedData.indexes;
            Object[] datas = snmpCachedData.datas;
            for (int i = indexes.length - 1; i > -1; --i) {
                MemoryManagerMXBean memoryManagerMXBean = (MemoryManagerMXBean)datas[i];
                if (memoryManagerMXBean != null) {
                    this.updateTreeMap(treeMap, o, memoryManagerMXBean, indexes[i], map);
                }
            }
        }
    }
}
